<?php

namespace Tests\Feature\User\Contact;

use SALESmanago\Controller\ContactAndEventTransferController;
use SALESmanago\Entity\ApiDoubleOptIn;
use SALESmanago\Exception\Exception;
use Tests\Feature\TestCaseFeature;
use SALESmanago\Entity\Contact\Contact;
use SALESmanago\Entity\Configuration;

class CreateContactTest extends TestCaseFeature
{
    /**
     * @var ContactAndEventTransferController
     */
    protected $transferController;

    /**
     * @var string - Salesmanago confirmation subscription email id
     */
    protected $emailId;

    /**
     * @return void
     * @throws Exception
     */
    public function setUp(): void
    {
        parent::setUp();

        $this->initConf();
        $this->transferController = new ContactAndEventTransferController(Configuration::getInstance());
        $this->emailId = getenv('emailId');
    }

    /**
     * Test of upsert contact with old useApiDoubleOptIn settings
     *
     * @return void
     * @throws Exception
     */
    public function testUpsertContactToSalesmanagoWithApiDoubleOptIntBasedOnThreeFields(): void
    {
        $ApiDoubleOptIn = new ApiDoubleOptIn();
        Configuration::getInstance()->setApiDoubleOptIn(
            $ApiDoubleOptIn
                ->setEnabled(true)
                ->setAccountId($this->faker->uuid)
                ->setTemplateId($this->faker->uuid)
        );

        $Contact = $this->generateContact();

        $Response = $this->transferController->transferContact($Contact);
        $this->assertNotEmpty($Response->getField('contactId'));
    }

    /**
     * Test Upset contact with apiDoubleOptInt with emilId and lang (newest option on sm api)
     *
     * @return void
     * @throws Exception
     */
    public function testUpsertContactToSalesmanagoWithApiDoubleOptIntBasedOnEmailId(): void
    {
        $ApiDoubleOptIn = new ApiDoubleOptIn();
        Configuration::getInstance()->setApiDoubleOptIn(
            $ApiDoubleOptIn
                ->setEnabled(true)
                ->setEmailId($this->emailId)
                ->setLang($this->faker->languageCode)
        );

        $Contact = $this->generateContact();

        $Response = $this->transferController->transferContact($Contact);
        $this->assertNotEmpty($Response->getField('contactId'));
    }

    /**
     * Generate Contact Entity with sample data
     *
     * @return Contact
     */
    protected function generateContact(): Contact
    {
        $Contact = new Contact();
        $Contact
            ->setEmail($this->faker->email)
            ->setCompany($this->faker->company)
            ->setFax($this->faker->phoneNumber)
            ->setName($this->faker->name)
            ->setExternalId($this->faker->uuid)
            ->setPhone($this->faker->phoneNumber)
            ->setAddress(
                ($Contact->getAddress())//return new Address Entity
                ->setCity($this->faker->city)
                    ->setCountry($this->faker->country)
                    ->setProvince($this->faker->word)
                    ->setZipCode($this->faker->postcode)
                    ->setStreetAddress($this->faker->streetAddress)
            )->setOptions(
                ($Contact->getOptions())
                    ->setIsSubscribes(true)//simulating subscribing process
                    ->setTags('NEWSLETTER_' . $this->faker->word)
                    ->setCreatedOn(time())
            );

        return $Contact;
    }
}