<?php

namespace Tests\Feature\User\Vendor;

use SALESmanago\Controller\User\Vendor\AccountController;
use SALESmanago\Entity\AbstractConfiguration;
use SALESmanago\Entity\UnionConfigurationEntity;
use SALESmanago\Entity\User;
use Tests\Feature\TestCaseFeature;
use SALESmanago\Exception\ApiV3Exception;
use SALESmanago\Exception\Exception as SalesmanagoException;

class LoginTest extends TestCaseFeature
{

    public function setUp(): void
    {
        parent::setUp();

        $reflection = new \ReflectionProperty(AbstractConfiguration::class, 'instances');
        $reflection->setAccessible(true);
        $reflection->setValue(null, []);
    }

    /**
     * Test if login of vendor is success
     *
     * @runTestsInSeparateProcesses
     * @return void
     * @throws ApiV3Exception
     * @throws SalesmanagoException
     */
    public function testLoginSuccess()
    {
        $userEmail = getenv('userEmail');
        $userPass  = getenv('userPass');

        $conf = UnionConfigurationEntity::getInstance();

        $AccountController = new AccountController($conf, null);

        $Response = $AccountController->login(
            (new User())
                ->setEmail($userEmail)
                ->setPass($userPass)
        );

        $this->assertNotEmpty(UnionConfigurationEntity::getInstance()->getApiV3Key());
        $this->assertNotEmpty(UnionConfigurationEntity::getInstance()->getEndpoint());
        $this->assertNotEmpty(UnionConfigurationEntity::getInstance()->getApiKey());
        $this->assertNotEmpty(UnionConfigurationEntity::getInstance()->getSha());

        $this->assertNotEmpty($Response->getField('conf')->getClientId());
        $this->assertNotEmpty($Response->getField('conf')->getApiV3Key());

        $this->assertEquals($Response->getField('conf')->getClientId(),
            UnionConfigurationEntity::getInstance()->getClientId()
        );

        $this->assertEquals($Response->getField('conf')->getApiV3Key(),
            UnionConfigurationEntity::getInstance()->getApiV3Key()
        );
    }

    protected function tearDown(): void
    {
        parent::tearDown();

        //clear configuration singleton
        $reflection = new \ReflectionProperty(AbstractConfiguration::class, 'instances');
        $reflection->setAccessible(true);
        $reflection->setValue(null, []);
    }
}